@@ -111,6 +111,8 @@ class PartTest
         void testeRectSelectionStartingOnLinks();
         void testCheckBoxReadOnly();
         void testCrashTextEditDestroy();
+        void testAnnotWindow();
+        void testAdditionalActionTriggers();
 
     private:
         void simulateMouseSelection(double startX, double startY, double endX, double endY, QWidget *target);
@@ -228,14 +230,14 @@ void PartTest::testForwardPDF()
     }
 
     const QString pdfResult = workDir.path() + QStringLiteral("/synctextest.pdf");
-    
+
     QVERIFY(QFile::exists(pdfResult));
-    
+
     QVERIFY( openDocument(&part, pdfResult) );
     part.m_document->setViewportPage(0);
     QCOMPARE(part.m_document->currentPage(), 0u);
     part.closeUrl();
-    
+
     QUrl u(QUrl::fromLocalFile(pdfResult));
     u.setFragment(QStringLiteral("src:100") + texDestination);
     part.openUrl(u);
@@ -1414,8 +1416,231 @@ void PartTest::testCrashTextEditDestroy()
     part.actionCollection()->action(QStringLiteral("view_toggle_forms"))->trigger();
 }
 
+void PartTest::testAnnotWindow()
+{
+    QVariantList dummyArgs;
+    Okular::Part part(nullptr, nullptr, dummyArgs);
+    QVERIFY(openDocument(&part, QStringLiteral(KDESRCDIR "data/file1.pdf")));
+    part.widget()->show();
+    part.widget()->resize(800, 600);
+    QVERIFY(QTest::qWaitForWindowExposed(part.widget()));
+
+    const int width =  part.m_pageView->horizontalScrollBar()->maximum() +
+                       part.m_pageView->viewport()->width();
+    const int height = part.m_pageView->verticalScrollBar()->maximum() +
+                       part.m_pageView->viewport()->height();
+
+    part.m_document->setViewportPage(0);
+
+    // wait for pixmap
+    QTRY_VERIFY(part.m_document->page(0)->hasPixmap(part.m_pageView));
+
+    QMetaObject::invokeMethod(part.m_pageView, "slotSetMouseNormal");
+
+    QCOMPARE(part.m_document->currentPage(), 0u);
+
+    // Create two distinct text annotations
+    Okular::Annotation * annot1 = new Okular::TextAnnotation();
+    annot1->setBoundingRectangle( Okular::NormalizedRect( 0.8, 0.1, 0.85, 0.15 ) );
+    annot1->setContents( QStringLiteral("Annot contents 111111") );
+
+    Okular::Annotation *annot2 = new Okular::TextAnnotation();
+    annot2->setBoundingRectangle( Okular::NormalizedRect(  0.8, 0.3, 0.85, 0.35 ) );
+    annot2->setContents( QStringLiteral("Annot contents 222222") );
+
+    // Add annot1 and annot2 to document
+    part.m_document->addPageAnnotation( 0, annot1 );
+    QTest::qWait(100);
+    part.m_document->addPageAnnotation( 0, annot2 );
+    QTest::qWait(100);
+    QVERIFY( part.m_document->page( 0 )->annotations().size() == 2 );
+
+    // Double click the first annotation to open its window (move mouse for visual feedback)
+    const NormalizedPoint annot1pt = annot1->boundingRectangle().center();
+    QTest::mouseMove(part.m_pageView->viewport(), QPoint(width * annot1pt.x, height * annot1pt.y));
+    QTest::mouseDClick(part.m_pageView->viewport(), Qt::LeftButton, Qt::NoModifier, QPoint(width * annot1pt.x, height * annot1pt.y));
+    QTRY_COMPARE( part.m_pageView->findChildren<QFrame *>("AnnotWindow").size(), 1 );
+    // Verify that the window is visible
+    QFrame* win1 = part.m_pageView->findChild<QFrame *>("AnnotWindow");
+    QVERIFY( !win1->visibleRegion().isEmpty() );
+
+    // Double click the second annotation to open its window (move mouse for visual feedback)
+    const NormalizedPoint annot2pt = annot2->boundingRectangle().center();
+    QTest::mouseMove(part.m_pageView->viewport(), QPoint(width * annot2pt.x, height * annot2pt.y));
+    QTest::mouseDClick(part.m_pageView->viewport(), Qt::LeftButton, Qt::NoModifier, QPoint(width * annot2pt.x, height * annot2pt.y));
+    QTRY_COMPARE( part.m_pageView->findChildren<QFrame *>("AnnotWindow").size(), 2 );
+    // Verify that the first window is hidden covered by the second, which is visible
+    QList<QFrame *> lstWin = part.m_pageView->findChildren<QFrame *>("AnnotWindow");
+    QFrame * win2;
+    if (lstWin[0] == win1) {
+        win2 = lstWin[1];
+    } else {
+        win2 = lstWin[0];
+    }
+    QVERIFY( win1->visibleRegion().isEmpty() );
+    QVERIFY( !win2->visibleRegion().isEmpty() );
+
+    // Double click the first annotation to raise its window (move mouse for visual feedback)
+    QTest::mouseMove(part.m_pageView->viewport(), QPoint(width * annot1pt.x, height * annot1pt.y));
+    QTest::mouseDClick(part.m_pageView->viewport(), Qt::LeftButton, Qt::NoModifier, QPoint(width * annot1pt.x, height * annot1pt.y));
+    // Verify that the second window is hidden covered by the first, which is visible
+    QVERIFY( !win1->visibleRegion().isEmpty() );
+    QVERIFY( win2->visibleRegion().isEmpty() );
+
+    // Move annotation window 1 to partially show annotation window 2
+    win1->move(QPoint(win2->pos().x(),  win2->pos().y() + 50));
+    // Verify that both windows are partially visible
+    QVERIFY( !win1->visibleRegion().isEmpty() );
+    QVERIFY( !win2->visibleRegion().isEmpty() );
+
+    // Click the second annotation window to raise it (move mouse for visual feedback)
+    auto widget = win2->window()->childAt(win2->mapTo(win2->window(), QPoint(10,  10)));
+    QTest::mouseMove(win2->window(), win2->mapTo(win2->window(), QPoint(10,  10)));
+    QTest::mouseClick(widget, Qt::LeftButton, Qt::NoModifier, widget->mapFrom(win2, QPoint(10,  10)));
+    QVERIFY( win1->visibleRegion().rects().count() == 3);
+    QVERIFY( win2->visibleRegion().rects().count() == 4);
 }
 
+// Helper for testAdditionalActionTriggers
+static void verifyTargetStates( const QString & triggerName,
+                                const QMap<QString, Okular::FormField *> &fields,
+                                bool focusVisible, bool cursorVisible, bool mouseVisible,
+                                int line)
+{
+    Okular::FormField *focusTarget = fields.value( triggerName + QStringLiteral ("_focus_target") );
+    Okular::FormField *cursorTarget = fields.value( triggerName + QStringLiteral ("_cursor_target") );
+    Okular::FormField *mouseTarget = fields.value( triggerName + QStringLiteral ("_mouse_target") );
+
+    QVERIFY( focusTarget );
+    QVERIFY( cursorTarget );
+    QVERIFY( mouseTarget );
+
+    QTRY_VERIFY2( focusTarget->isVisible() == focusVisible,
+                  QStringLiteral ("line: %1 focus for %2 not matched. Expected %3 Actual %4").
+                  arg( line ).arg( triggerName ).arg( focusTarget->isVisible() ).arg( focusVisible ).toUtf8().constData() );
+    QTRY_VERIFY2( cursorTarget->isVisible() == cursorVisible,
+                  QStringLiteral ("line: %1 cursor for %2 not matched. Actual %3 Expected %4").
+                  arg( line ).arg( triggerName ).arg( cursorTarget->isVisible() ).arg( cursorVisible ).toUtf8().constData() );
+    QTRY_VERIFY2( mouseTarget->isVisible() == mouseVisible,
+                  QStringLiteral ("line: %1 mouse for %2 not matched. Expected %3 Actual %4").
+                  arg( line ).arg( triggerName ).arg( mouseTarget->isVisible() ).arg( mouseVisible ).toUtf8().constData() );
+}
+
+void PartTest::testAdditionalActionTriggers()
+{
+#ifndef HAVE_POPPLER_0_65
+    return;
+#endif
+    const QString testFile = QStringLiteral( KDESRCDIR "data/additionalFormActions.pdf" );
+    Okular::Part part( nullptr, nullptr, QVariantList() );
+    part.openDocument( testFile );
+    part.widget()->resize(800, 600);
+
+    part.widget()->show();
+    QVERIFY( QTest::qWaitForWindowExposed( part.widget() ) );
+
+    QMap<QString, Okular::FormField *> fields;
+    // Field names in test document are:
+    // For trigger fields: tf, cb, rb, dd, pb
+    // For target fields: <trigger_name>_focus_target, <trigger_name>_cursor_target,
+    // <trigger_name>_mouse_target
+    const Okular::Page* page = part.m_document->page( 0 );
+    for ( Okular::FormField *ff: page->formFields() )
+    {
+        fields.insert( ff->name(), static_cast< Okular::FormField* >( ff ) );
+    }
+
+    // Verify that everything is set up.
+    verifyTargetStates( QStringLiteral( "tf" ), fields, true, true, true, __LINE__ );
+    verifyTargetStates( QStringLiteral( "cb" ), fields, true, true, true, __LINE__ );
+    verifyTargetStates( QStringLiteral( "rb" ), fields, true, true, true, __LINE__ );
+    verifyTargetStates( QStringLiteral( "dd" ), fields, true, true, true, __LINE__ );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, true, true, true, __LINE__ );
+
+    const int width = part.m_pageView->horizontalScrollBar()->maximum() +
+                      part.m_pageView->viewport()->width();
+    const int height = part.m_pageView->verticalScrollBar()->maximum() +
+                       part.m_pageView->viewport()->height();
+
+    part.m_document->setViewportPage( 0 );
+
+    // wait for pixmap
+    QTRY_VERIFY( part.m_document->page( 0 )->hasPixmap( part.m_pageView) );
+
+    part.actionCollection()->action( QStringLiteral( "view_toggle_forms" ) )->trigger();
+
+    QPoint tfPos( width * 0.045, height * 0.05 );
+    QPoint cbPos( width * 0.045, height * 0.08 );
+    QPoint rbPos( width * 0.045, height * 0.12 );
+    QPoint ddPos( width * 0.045, height * 0.16 );
+    QPoint pbPos( width * 0.045, height * 0.26 );
+
+    // Test text field
+    auto widget = part.m_pageView->viewport()->childAt( tfPos );
+    QVERIFY( widget );
+
+    QTest::mouseMove( part.m_pageView->viewport(), QPoint( tfPos ));
+    verifyTargetStates( QStringLiteral( "tf" ), fields, true, false, true, __LINE__ );
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "tf" ), fields, false, false, false, __LINE__ );
+    QTest::mouseRelease( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "tf" ), fields, false, false, true, __LINE__ );
+
+    // Checkbox
+    widget = part.m_pageView->viewport()->childAt( cbPos );
+    QVERIFY( widget );
+
+    QTest::mouseMove( part.m_pageView->viewport(), QPoint( cbPos ) );
+    verifyTargetStates( QStringLiteral( "cb" ), fields, true, false, true, __LINE__ );
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "cb" ), fields, false, false, false, __LINE__ );
+    // Confirm that the textfield no longer has any invisible
+    verifyTargetStates( QStringLiteral( "tf" ), fields, true, true, true, __LINE__ );
+    QTest::mouseRelease( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "cb" ), fields, false, false, true, __LINE__ );
+
+    // Radio
+    widget = part.m_pageView->viewport()->childAt( rbPos );
+    QVERIFY( widget );
+
+    QTest::mouseMove( part.m_pageView->viewport(), QPoint( rbPos ) );
+    verifyTargetStates( QStringLiteral( "rb" ), fields, true, false, true, __LINE__ );
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "rb" ), fields, false, false, false, __LINE__ );
+    QTest::mouseRelease( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "rb" ), fields, false, false, true, __LINE__ );
+
+    // Dropdown
+    widget = part.m_pageView->viewport()->childAt( ddPos );
+    QVERIFY( widget );
+
+    QTest::mouseMove( part.m_pageView->viewport(), QPoint( ddPos ) );
+    verifyTargetStates( QStringLiteral( "dd" ), fields, true, false, true, __LINE__ );
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "dd" ), fields, false, false, false, __LINE__ );
+    QTest::mouseRelease( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "dd" ), fields, false, false, true, __LINE__ );
+
+    // Pushbutton
+    widget = part.m_pageView->viewport()->childAt( pbPos );
+    QVERIFY( widget );
+
+    QTest::mouseMove( part.m_pageView->viewport(), QPoint( pbPos ) );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, true, false, true, __LINE__ );
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, false, false, false, __LINE__ );
+    QTest::mouseRelease( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, false, false, true, __LINE__ );
+
+    // Confirm that a mouse release outside does not trigger the show action.
+    QTest::mousePress( widget, Qt::LeftButton, Qt::NoModifier, QPoint( 5, 5 ) );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, false, false, false, __LINE__ );
+    QTest::mouseRelease( part.m_pageView->viewport(), Qt::LeftButton, Qt::NoModifier, tfPos );
+    verifyTargetStates( QStringLiteral( "pb" ), fields, false, false, false, __LINE__ );
+}
+
+} // namespace Okular
+
 int main(int argc, char *argv[])
 {
     // Force consistent locale
