@@ -9,4 +9,4 @@
 
 #include "debug_p.h"
 
-Q_LOGGING_CATEGORY(OkularCoreDebug, "org.kde.okular.core")
+Q_LOGGING_CATEGORY(OkularCoreDebug, "org.kde.okular.core", QtWarningMsg)
