@@ -311,7 +311,7 @@ QPair<Okular::Movie*, Okular::EmbeddedFile*> createMovieFromPopplerRichMedia( co
 /**
  * Note: the function will take ownership of the popplerLink object.
  */
-Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
+Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink, bool deletePopplerLink = true)
 {
     if (!popplerLink)
         return nullptr;
@@ -327,8 +327,6 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
     const Poppler::LinkRendition *popplerLinkRendition;
     Okular::DocumentViewport viewport;
 
-    bool deletePopplerLink = true;
-
     switch(popplerLink->linkType())
     {
         case Poppler::Link::None:
@@ -384,6 +382,17 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 
         case Poppler::Link::Rendition:
         {
+            if (!deletePopplerLink)
+            {
+                // If links should not be deleted it probably means that they
+                // are part of a nextActions chain. There is no support
+                // to resolveMediaLinkReferences on nextActions. It would also
+                // be neccessary to ensure that resolveMediaLinkReferences does
+                // not delete the Links which are part of a nextActions list
+                // to avoid a double deletion.
+                qCDebug(OkularPdfDebug) << "parsing rendition link without deletion is not supported. Action chain might be broken.";
+                break;
+            }
             deletePopplerLink = false; // we'll delete it inside resolveMediaLinkReferences() after we have resolved all references
 
             popplerLinkRendition = static_cast<const Poppler::LinkRendition *>( popplerLink );
@@ -420,6 +429,12 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 
         case Poppler::Link::Movie:
         {
+            if (!deletePopplerLink)
+            {
+                // See comment above in Link::Rendition
+                qCDebug(OkularPdfDebug) << "parsing movie link without deletion is not supported. Action chain might be broken.";
+                break;
+            }
             deletePopplerLink = false; // we'll delete it inside resolveMediaLinkReferences() after we have resolved all references
 
             popplerLinkMovie = static_cast<const Poppler::LinkMovie *>( popplerLink );
@@ -446,8 +461,34 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
             link = movieAction;
         }
         break;
+
+#ifdef HAVE_POPPLER_0_64
+        case Poppler::Link::Hide:
+        {
+            const Poppler::LinkHide * l = static_cast<const Poppler::LinkHide *>( popplerLink );
+            QStringList scripts;
+            for ( const QString &target: l->targets() )
+            {
+                scripts << QStringLiteral( "getField(\"%1\").hidden = %2;" ).arg( target ).arg( l->isShowAction() ? QLatin1String( "false" ) : QLatin1String( "true" ) );
+            }
+            link = new Okular::ScriptAction( Okular::JavaScript, scripts.join( QLatin1Char( '\n' ) ) );
+        }
+        break;
+#endif
     }
 
+#ifdef HAVE_POPPLER_0_64
+    if (link)
+    {
+        QVector< Okular::Action * > nextActions;
+        for ( const Poppler::Link *nl : popplerLink->nextLinks() )
+        {
+            nextActions << createLinkFromPopplerLink( nl, false );
+        }
+        link->setNextActions( nextActions );
+    }
+#endif
+
     if ( deletePopplerLink )
         delete popplerLink;
 
